<?php

/*
 * Plugin: Gerymania
 * ~~~~~~~~~~~~~~~~~
 * This Plugin works with the well known Gerymania from the Server-Controller TM Gery.
 * Gerymania is similar to Dedimania (a world records database), but only for the
 * Gamemode 'Stunts'.
 * ----------------------------------------------------------------------------------
 * Author:		undef.de
 * Version:		0.9.6
 * Date:		2012-02-17
 * Copyright:		2010 - 2012 by undef.de
 * Home:		http://labs.undef.de/
 * System:		XAseco/1.14+
 * Game:		Trackmania Forever (TMF) only
 * ----------------------------------------------------------------------------------
 *
 * LICENSE: This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 *
 * ----------------------------------------------------------------------------------
 *
 * Dependencies: none
 */

Aseco::registerEvent('onSync',				'gery_onSync');
Aseco::registerEvent('onShutdown',			'gery_onShutdown');
Aseco::registerEvent('onNewChallenge2',			'gery_onNewChallenge2');
Aseco::registerEvent('onPlayerFinish',			'gery_onPlayerFinish');
Aseco::registerEvent('onEverySecond',			'gery_onEverySecond');


global $gery_config, $gery_db;
$gery_db = false;

/* Just a note for my editor -> utf-8 äöüß */

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onSync
function gery_onSync ($aseco) {
	global $gery_config;


	// Check for the right XAseco-Version
	$xaseco_min_version = '1.14';
	if ( defined('XASECO_VERSION') ) {
		if ( version_compare(XASECO_VERSION, $xaseco_min_version, '<') ) {
			trigger_error('[plugin.gerymania.php] Not supported XAseco version ('. XASECO_VERSION .')! Please update to min. version '. $xaseco_min_version .'!', E_USER_ERROR);
		}
	}
	else {
		trigger_error('[plugin.gerymania.php] Can not identify the System, "XASECO_VERSION" is unset! This plugin runs only with XAseco/'. $xaseco_min_version .'+', E_USER_ERROR);
	}

	if ($aseco->server->getGame() != 'TMF') {
		trigger_error('[plugin.gerymania.php] This plugin supports only TMF, can not start with a "'. $aseco->server->getGame() .'" Dedicated-Server!', E_USER_ERROR);
	}

	// Read Configuration
	if (!$cfg = $aseco->xml_parser->parseXML('gerymania.xml')) {
		trigger_error('[plugin.gerymania.php] Could not read/parse config file "gerymania.xml"!', E_USER_ERROR);
	}
	$cfg = $cfg['GERYMANIA'];


	$gery_config['recs_in_window']			= ((strtoupper($cfg['RECS_IN_WINDOW'][0]) == 'TRUE') ? true : false);

	$gery_config['messages']['record_new']		= $cfg['MESSAGES'][0]['RECORD_NEW'][0];
	$gery_config['messages']['record_equal']	= $cfg['MESSAGES'][0]['RECORD_EQUAL'][0];
	$gery_config['messages']['record_new_rank']	= $cfg['MESSAGES'][0]['RECORD_NEW_RANK'][0];
	$gery_config['messages']['record_first']	= $cfg['MESSAGES'][0]['RECORD_FIRST'][0];

	$gery_config['request_authenticate']		= 'VG1HZXJ5UmVjIFhBU0VDTyB2MjAxMC0xMC0yMCBodHRwOi8vdHJhY2ttYW5pYS5nZXJ5LnBlcnNvLnNmci5mcg==';
	$gery_config['remote_call_url']			= 'http://asbarak.chez-alice.fr/TMGery_remote.php';

	$gery_config['refresh_interval']		= 10;
	$gery_config['retry']				= 0;

	$gery_config['debug']				= false;
	$gery_config['version']				= '0.9.6';


	// Register this to the global version pool (for up-to-date checks)
	$aseco->plugin_versions[] = array(
		'plugin'	=> 'plugin.gerymania.php',
		'author'	=> 'undef.de',
		'version'	=> $gery_config['version']
	);


	// Debug?
	if ($gery_config['debug'] == true) {
		// Create debugging log file
		$filename = './debug_gerymania.log';
		if ( !$fh = fopen($filename, "a", 0) ) {
			trigger_error('[plugin.gerymania.php] Could not create debugging log file "'. $filename .'"!', E_USER_ERROR);
		}
		else {
			$gery_config['debug_logfile'] = $fh;
		}
	}

	unset($cfg);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onShutdown
function gery_onShutdown ($aseco) {
	global $gery_config;


	// Debug?
	if ($gery_config['debug'] == true) {
		// Close debugging log file
		fclose($gery_config['debug_logfile']);
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onNewChallenge2
function gery_onNewChallenge2 ($aseco, $challenge_item) {
	global $gery_config, $gery_db;


	// Debug?
	if ($gery_config['debug'] == true) {
		// Write a notice
		fwrite($gery_config['debug_logfile'], 'gery_onNewChallenge2() called at Gamemode "'. $aseco->server->gameinfo->mode .'" for Challenge uid "'. $aseco->server->challenge->uid .'"' . "\n");
	}

	// Destroy the global records array
	$gery_db = false;

	// Bail out immediately if not 'Stunts'
	if ($aseco->server->gameinfo->mode != 4) {
		return;
	}

	// Bail out without challenge UID
	if ( !$aseco->server->challenge->uid ) {
		return;
	}


	// Get the current records for this challenge
	$param = array(
		'uid'		=> $aseco->server->challenge->uid,
		'server'	=> $aseco->server->serverlogin
	);
	$recs = gery_sendRequest(
		$gery_config['remote_call_url'],
		$gery_config['request_authenticate'],
		'gettop',
		$param
	);

	// Debug?
	if ($gery_config['debug'] == true) {

		ob_start();
		var_dump($recs);
		$dump = ob_get_clean();

		// Write a notice
		fwrite($gery_config['debug_logfile'], 'gery_sendRequest() results (dump): '. $dump . "\n");
	}

	// Did this challenge have saved gerymania records?
	if (count($recs[0]) > 0) {

		// Init array
		$gery_db = array();

		// Save the records
		for ($i = 0; $i < count($recs[0]); $i ++) {
			$recs[0][$i]['nick'] = str_replace("\'", "'", $recs[0][$i]['nick']);
			$gery_db[] = $recs[0][$i];
		}
	}
	else {
		// Setup empty array
		$gery_db = array();
	}

	// Release 'loaded' event
	$aseco->releaseEvent('onGerymaniaRecordsLoaded', $gery_db);

	// Debug?
	if ($gery_config['debug'] == true) {

		ob_start();
		var_dump($gery_db);
		$dump = ob_get_clean();

		// Write a notice
		fwrite($gery_config['debug_logfile'], 'gery_onNewChallenge2() $gery_db (dump) after work: '. $dump . "\n########################################\n");
	}

}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function gery_onPlayerFinish ($aseco, $finish_item) {
	global $gery_config, $gery_db;


	// If no actual finish, bail out immediately
	if ($finish_item->score == 0) {
		return;
	}

	// Get current Gamemode
	$gamemode = $aseco->server->gameinfo->mode;

	// Check if the current GameMode is 'Stunts', do nothing if not!
	if ($gamemode == 4) {

		// Purge login
		$login = gery_purgelogin($finish_item->player->login);

		// Use stripColors() from XASECO!
		$nickname = stripColors($finish_item->player->nickname);

		// Mark as 'no new record'
		$new_record = false;


		// Did this Player has already a gerymania record?
		$current_record = array();
		for ($i = 0; $i < count($gery_db); $i ++) {
			if ($gery_db[$i]['login'] == $login) {
				$current_record['login']	= $gery_db[$i]['login'];
				$current_record['nick']		= $gery_db[$i]['nick'];
				$current_record['score']	= $gery_db[$i]['score'];
				$current_record['rank']		= $i;
				break;
			}
		}
		if ( isset($current_record['login']) ) {
			// Yes, Player has already a gerymania record

			// Did he improved or equaled his record?
			if ($finish_item->score > $current_record['score']) {

				// Did the Player get an new rank?
				$rank = -1;
				for ($i = 0; $i < count($gery_db); $i ++) {
					if ($finish_item->score > $gery_db[$i]['score']) {
						$rank = $i;		// Add them at this rank
						break;
					}
					else if ($finish_item->score == $gery_db[$i]['score']) {
						$rank = ($i + 1);	// Add them behind current rank
						break;
					}
				}

				if ($gery_config['debug'] == true) {
					fwrite($gery_config['debug_logfile'], 'gery_onPlayerFinish() Player "'. $finish_item->player->login .'" get rank "'. ($rank+1) .'" was before "'. ($current_record['rank']+1) .'"' . "\n########################################\n");
				}

				if ( ($rank < $current_record['rank']) && ($rank != -1) ) {
					// Yes new gerymania rank

					// Create the new rank entry
					$new_entry = array(
						'login'	=> $login,
						'nick'	=> $finish_item->player->nickname,
						'score'	=> $finish_item->score,
					);

					$new_recs = array();
					for ($i = 0; $i < count($gery_db); $i ++) {
						if ($rank == $i) {

							// Insert new rank
							$new_recs[] = $new_entry;

							// Add the rest (without the last one) behind of the new rank
							for ($j = $i; $j < (count($gery_db) - 1); $j ++) {
								// Add only other Player, not current!
								if ($login != $gery_db[$j]['login']) {
									$new_recs[] = $gery_db[$j];
								}
							}

							// Do a player new rank message
							$message = formatText(
								$gery_config['messages']['record_new_rank'],
								$nickname,
								$i+1,
								'Score',
								$finish_item->score,
								$i+1,
								'+'. ($finish_item->score - $current_record['score'])
							);
							gery_sendChatMessage($aseco, $message);

							// Release 'new gerymania record' event
							$aseco->releaseEvent('onGerymaniaRecord', $new_entry);

							// Stop here
							break;
						}
						else {
							$new_recs[] = $gery_db[$i];
						}
					}

					// Set the new array
					$gery_db = $new_recs;

				}
				else {
					// No new gerymania rank, but improved score

					// Do a player secured his/her gerymania record message
					$message = formatText(
						$gery_config['messages']['record_new'],
						$nickname,
						$i+1,
						'Score',
						$finish_item->score,
						$i+1,
						'+'. ($finish_item->score - $current_record['score'])
					);
					gery_sendChatMessage($aseco, $message);

					// Save the new gerymania score
					$gery_db[$current_record['rank']]['score'] = $finish_item->score;

					// Release 'new gerymania record' event
					$aseco->releaseEvent('onGerymaniaRecord', $gery_db[$current_record['rank']]);
				}

				// Mark for sending
				$new_record = true;

			}
			else if ($finish_item->score == $current_record['score']) {

				// Do a player equaled his/her gerymania record message
				$message = formatText(
					$gery_config['messages']['record_equal'],
					$nickname,
					$i+1,
					'Score',
					$finish_item->score
				);
				gery_sendChatMessage($aseco, $message);

			}
		}
		else {
			// Player did not have a gerymania record, let´s see if he beat one other record

			// Create the new rank entry
			$new_entry = array(
				'login'	=> $login,
				'nick'	=> $finish_item->player->nickname,
				'score'	=> $finish_item->score,
			);

			$new_recs = array();
			for ($i = 0; $i < count($gery_db); $i ++) {
				if ($finish_item->score > $gery_db[$i]['score']) {

					// Insert new rank
					$new_recs[] = $new_entry;

					// Add the rest (without the last one) behind of the new rank
					for ($j = $i; $j < (count($gery_db) - 1); $j ++) {
						$new_recs[] = $gery_db[$j];
					}

					// Do a player drove first record message
					$message = formatText(
						$gery_config['messages']['record_first'],
						$nickname,
						$i+1,
						'Score',
						$finish_item->score
					);

					gery_sendChatMessage($aseco, $message);

					// Release 'new gerymania record' event
					$aseco->releaseEvent('onGerymaniaRecord', $new_entry);

					// Mark for sending
					$new_record = true;

					break;
				}
				else {
					$new_recs[] = $gery_db[$i];
				}
			}

			// Set the new array
			$gery_db = $new_recs;

		}


		// Send an new record to the gerymania database?
		if ($new_record == true) {

			// Send the new record
			$param = array(
				'uid'		=> $finish_item->challenge->uid,
				'login'		=> $login,
				'nick'		=> $finish_item->player->nickname,
				'score'		=> intval($finish_item->score),
				'server'	=> $aseco->server->serverlogin,
				'gamemode'	=> intval($gamemode)
			);
			$response = gery_sendRequest(
				$gery_config['remote_call_url'],
				$gery_config['request_authenticate'],
				'playerfinish',
				$param
			);

			// Error?
			if ($response[0] != true) {
				$aseco->console('[plugin.gerymania.php] Error at save to database: '. $response[0]);

				// Debug?
				if ($gery_config['debug'] == true) {

					ob_start();
					var_dump($response);
					$dump = ob_get_clean();

					fwrite($gery_config['debug_logfile'], 'gery_onPlayerFinish() gery_sendRequest() $response results: '. $dump ."\n########################################\n");
				}
			}
		}
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

function gery_sendChatMessage ($aseco, $message) {
	global $gery_config;


	// Show message
	if ( isset($message) ) {
		if ( ($gery_config['recs_in_window'] == true) && ( function_exists('send_window_message') ) ) {
			send_window_message($aseco, $message, false);
		}
		else {
			$aseco->client->query('ChatSendServerMessage', $aseco->formatColors($message));
		}
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// Stolen and adapted from plugin.geryrec.php Version 3.4
function gery_sendRequest ($url, $auth, $service_name, $arguments = array()) {
	global $aseco, $gery_config;


	// Debug?
	if ($gery_config['debug'] == true) {

		ob_start();
		var_dump($arguments);
		$dump = ob_get_clean();

		fwrite($gery_config['debug_logfile'], 'gery_sendRequest() called with $arguments: '. $dump ."\n########################################\n");
	}

	$data = http_build_query($arguments);
	$opts = array (
		'http' => array (
			'method' => 'POST',
			'header' =>	"x-auth: $auth\r\n" .
					"x-service-name: $service_name\r\n".
					"Content-type: application/x-www-form-urlencoded\r\n".
					"Content-Length: ". strlen($data) ."\r\n",
			'content' => $data,
			'timeout' => 10.0
		)
	);

	$context = stream_context_create($opts);

//	$t1 = microtime(true);
	$content = @file_get_contents($url, false, $context);
//	$aseco->console('[plugin.gerymania.php] access time: '. (microtime(true)-$t1));

	if ($content === false) {
		$aseco->console('[plugin.gerymania.php] Error at connection to database: server timeout');

		// Set retry
		$gery_config['retry'] = (time() + $gery_config['refresh_interval']);

		if ($gery_config['debug'] == true) {
			fwrite($gery_config['debug_logfile'], 'gery_sendRequest() Error at connection to database: server timeout' . "\n########################################\n");
		}

	}
	else {
		$result = unserialize($content);

		if ($result === false) {
			$aseco->console('[plugin.gerymania.php] Error at connection to database: unserialize error');

			// Set retry
			$gery_config['retry'] = (time() + $gery_config['refresh_interval']);

			// Debug?
			if ($gery_config['debug'] == true) {
				fwrite($gery_config['debug_logfile'], 'gery_sendRequest() Error at connection to database: unserialize error' . "\n########################################\n");
			}
		}
		else if (! is_array($result) || !( array_key_exists('exception', $result) || array_key_exists('return', $result) ) ) {
			$aseco->console('[plugin.gerymania.php] Error at connection to database: unknown result');

			// Set retry
			$gery_config['retry'] = (time() + $gery_config['refresh_interval']);

			// Debug?
			if ($gery_config['debug'] == true) {
				fwrite($gery_config['debug_logfile'], 'gery_sendRequest() Error at connection to database: unknown result' . "\n########################################\n");
			}
		}
	}

	if ( !empty($result['exception']) ) {
		$aseco->console('[plugin.gerymania.php] Error at connection to database: '. $result['exception']);

		// Set retry
		$gery_config['retry'] = (time() + $gery_config['refresh_interval']);

		// Debug?
		if ($gery_config['debug'] == true) {
			fwrite($gery_config['debug_logfile'], 'gery_sendRequest() Error at connection to database: '. $result['exception'] . "\n########################################\n");
		}
	}
	else {
		return $result['return'];
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// called @ onEverySecond
function gery_onEverySecond ($aseco) {
	global $gery_config;


	if ( ($gery_config['retry'] != 0) && (time() >= $gery_config['retry']) ) {
		$gery_config['retry'] = 0;
		gery_onNewChallenge2($aseco, null);
	}
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// Stolen from tm_gery.php Version 2010-07-16
function gery_purgelogin ($login) {


	$tmp1 = strrchr($login, "/");
	//echo "tmp1=".$tmp1."\n\r";

	$tmp2 = str_replace($tmp1, '', $login);
	//echo "tmp2=".$tmp2."\n\r";

	$tmp = gery_stripColors($tmp2, 1);
	//  echo "purgelogin=".$tmp."\n\r";

	return strtolower($tmp);
}

/*
#///////////////////////////////////////////////////////////////////////#
#									#
#///////////////////////////////////////////////////////////////////////#
*/

// Stolen from tm_gery.php Version 2010-07-16
function gery_stripColors ($str, $strip){
   	if ($strip==0) return $str;
	$str = "a".$str;
	$str = str_replace('$i','',$str);
	$str = str_replace('$I','',$str);
	$str = str_replace('$w','',$str);
	$str = str_replace('$W','',$str);
	$str = str_replace('$s','',$str);
	$str = str_replace('$S','',$str);
	$str = str_replace('$z','',$str);
	$str = str_replace('$Z','',$str);
	$str = str_replace('$n','',$str);
	$str = str_replace('$N','',$str);
	$str = str_replace('$m','',$str);
	$str = str_replace('$M','',$str);
	$str = str_replace('$g','',$str);
	$str = str_replace('$G','',$str);
	$str = str_replace('$o','',$str);
	$str = str_replace('$O','',$str);
//	$str = str_replace("\n",' ',$str);
//	$str = str_replace("\r",' ',$str);

	$found = true;
	while ($found) {
		$found = false;
		$start_a = strpos($str, '$l[');
		if ( $start_a > 0) {
			$end_a = strpos($str, ']');
			if ( $end_a > $start_a) {
				$tmp = substr($str, $start_a, $end_a - $start_a +1);
				$str = str_replace ($tmp, '', $str);
				$found = true;
			}
		}
		$start_a = strpos($str, '$L[');
		if ( $start_a > 0) {
			$end_a = strpos($str, ']');
			if ( $end_a > $start_a) {
				$tmp = substr($str, $start_a, $end_a - $start_a +1);
				$str = str_replace ($tmp, '', $str);
				$found = true;
			}
		}
		$start_a = strpos($str, '$h[');
		if ( $start_a > 0) {
			$end_a = strpos($str, ']');
			if ( $end_a > $start_a) {
				$tmp = substr($str, $start_a, $end_a - $start_a +1);
				$str = str_replace ($tmp, '', $str);
				$found = true;
			}
		}
		$start_a = strpos($str, '$H[');
		if ( $start_a > 0) {
			$end_a = strpos($str, ']');
			if ( $end_a > $start_a) {
				$tmp = substr($str, $start_a, $end_a - $start_a +1);
				$str = str_replace ($tmp, '', $str);
				$found = true;
			}
		}
	}

	$str = str_replace('$$','$',$str);
	$str = str_replace('$l','',$str);
	$str = str_replace('$L','',$str);
	$str = str_replace('$h','',$str);
	$str = str_replace('$H','',$str);

	$i=0;
	while($start = strpos($str,'$')){
		$i++;
		$color = substr($str, $start, 4);
		$str = str_replace($color, '', $str);
	}
	return substr($str,1);
}

?>
